require 'dotenv'
require 'json'
require 'logger'
require 'multi_json'
require 'restless_router'

Dotenv.load

require File.expand_path('../../nucleo/client/version', __FILE__)

require File.expand_path('../../nucleo/logger', __FILE__)
require File.expand_path('../../nucleo/configuration', __FILE__)
require File.expand_path('../../nucleo/connection', __FILE__)
require File.expand_path('../../nucleo/requests', __FILE__)
require File.expand_path('../../nucleo/response', __FILE__)
require File.expand_path('../../nucleo/errors', __FILE__)

require File.expand_path('../../nucleo/utilities', __FILE__)

require File.expand_path('../../nucleo/models', __FILE__)

require File.expand_path('../configuration', __FILE__)


module Nucleo
  class Client

    # Class accessor methods to be
    # utilities throughout the client.
    class << self
      attr_accessor :configuration
      attr_accessor :routes
    end

    # Create a default configuration to use
    #
    # @return [Nucleo::Configuration]
    def self.configuration
      @configuration ||= Nucleo::Configuration.new
    end

    # Specify configuration options. This will be
    # applied to our memoized Configuration.
    #
    # @return [Nucleo::Configuration]
    def self.configure
      yield(self.configuration)
    end

    # Helper method to access the Connection object
    #
    # @return [Nucleo::Connection]
    def self.connection
      @connection ||= Nucleo::Connection.new(url: self.configuration.api_host) do |builder|
        builder.use(:gzip)

        builder.response(:json, content_type: /\bjson$/)

        builder.response(:logger, self.configuration.request_logger)
        builder.response(:detailed_logger, self.configuration.detailed_logger)

        builder.request(:retry, max: 10, interval: 1, interval_randomness: 0.5, backoff_factor: 2)

        builder.options[:timeout] = 600

        builder.adapter(Nucleo::Connection.default_adapter)
      end

      # Inject the Authorization via the API key
      @connection.headers['x-api-key'] = self.configuration.access_token

      # Merge in the default headers
      @connection.headers.merge!(self.configuration.connection_options[:headers])

      @connection
    end

    # Helper method to perform a GET request
    #
    # @return [Nucleo::Response] Faraday Response Delegator
    def self.get(url, data={}, headers={})
      request = self.connection.get(url, data, headers)

      Nucleo::Response.new(request)
    end

    # Helper method to perform a HEAD request
    #
    # @return [Nucleo::Response] Faraday Response Delegator
    def self.head(url, data={}, headers={})
      request = self.connection.head(url, data, headers)

      Nucleo::Response.new(request)
    end

    # Helper method to perform an OPTIONS request
    #
    # @return [Nucleo::Response] Faraday Response Delegator
    def self.options(url, headers={})
      request = self.connection.http_options(url, nil, headers)

      Nucleo::Response.new(request)
    end

    # Helper method to perform an POST request
    #
    # @return [Nucleo::Response] Faraday Response Delegator
    def self.post(url, data={}, headers={})
      request = self.connection.post(url, data, headers)

      Nucleo::Response.new(request)
    end

    # Helper method to perform an PUT request
    #
    # @return [Nucleo::Response] Faraday Response Delegator
    def self.put(url, data={}, headers={})
      request = self.connection.put(url, data, headers)

      Nucleo::Response.new(request)
    end

    # Helper method to perform an PATCh request
    #
    # @return [Nucleo::Response] Faraday Response Delegator
    def self.patch(url, data={}, headers={})
      request = self.connection.patch(url, data, headers)

      Nucleo::Response.new(request)
    end

    # Helper method to perform an DELETE request
    #
    # @return [Nucleo::Response] Faraday Response Delegator
    def self.delete(url, data={}, headers={})
      request = self.connection.delete(url, data, headers)

      Nucleo::Response.new(request)
    end

    # Define the API routes. This will return a collection of
    # available routes.
    def self.routes
      return @routes if @routes

      @routes = RestlessRouter::Routes.new

      @routes.add_route(RestlessRouter::Route.new('analyzer', ("%sanalyzer" % [self.api_host])))
      @routes.add_route(RestlessRouter::Route.new('analyzer-site-page', ("%sanalyzer/sites/{site_id}/pages/{id}" % [self.api_host]), templated: true))
      @routes.add_route(RestlessRouter::Route.new('sites', ("%ssites" % [self.api_host])))
      @routes.add_route(RestlessRouter::Route.new('site', ("%ssites/{id}{?includes,metrics.per_page,metrics.page,alerts.per_page,alerts.page,pages.per_page,pages.page,changes.per_page,changes.page}" % [self.api_host]), templated: true))
      @routes.add_route(RestlessRouter::Route.new('site-process', ("%ssites/{id}/process" % [self.api_host]), templated: true))
      @routes.add_route(RestlessRouter::Route.new('site-pages', ("%ssites/{id}/pages" % [self.api_host]), templated: true))
      @routes.add_route(RestlessRouter::Route.new('site-page-detail', ("%ssites/{site_id}/pages/{id}" % [self.api_host]), templated: true))
      @routes.add_route(RestlessRouter::Route.new('site-page-alerts', ("%ssites/{site_id}/alerts/{type}" % [self.api_host]), templated: true))
      @routes.add_route(RestlessRouter::Route.new('site-feed', ("%ssites/{id}/feed{?max_feed_items}" % [self.api_host]), templated: true))
      @routes.add_route(RestlessRouter::Route.new('site-page-changes', ("%ssites/{site_id}/pages/{page_id}/changes" % [self.api_host]), templated: true))
      @routes.add_route(RestlessRouter::Route.new('site-changes-batch', ("%ssites/{site_id}/changes/batch" % [self.api_host]), templated: true))
      @routes.add_route(RestlessRouter::Route.new('site-changes', ("%ssites/{site_id}/changes" % [self.api_host]), templated: true))
      @routes.add_route(RestlessRouter::Route.new('util-checkjs', ("%sutil/checkjs?url={url}" % [self.api_host]), templated: true))

      @routes
    end

    # Helper method to return the API host
    #
    # @return [String] API Base URL
    def self.api_host
      self.configuration.api_host
    end
  end
end
