require File.expand_path('../page', __FILE__)

module Nucleo
  module Models
    class Feed
      include Comparable

      # Return an instance of the Feed domain model
      #
      # @param attributes [Hash]
      #
      # @return [Nucleo::Models::Feed]
      def initialize(attributes={})
        @attributes = attributes
      end

      def self.for_record(attributes={})
        const_name = ['nucleo', 'models', 'feed_types', attributes['category'], attributes['type']].map(&:camelcase).join('::')

        record = nil

        begin
          record = Object.const_get(const_name).new(attributes)
        rescue
          Nucleo::Client.configuration.logger.fatal("Could not instantiate %s" % [const_name])
        end

        record
      end

      # Sort by the executed at attribute
      def <=>(other)
        self.executed_at <=> other.executed_at
      end

      # Returns the page id
      #
      # @return [String]
      def page_id
        @attributes['page_id']
      end

      # Returns the page, if present
      #
      # @note: We have to inject the ID back in as it's
      # not present in the `page` response.
      #
      # @return [Nucleo::Models::Page,nil]
      def page
        value = @attributes['page']

        return nil if value.nil? || value.empty?

        value.merge!('id' => self.page_id)

        Nucleo::Models::Page.new(value)
      end

      # Returns true if there is an attached page
      #
      # @return [Boolean]
      def page?
        !self.page.nil?
      end

      # Returns the page recommendation ID
      #
      # @return [String]
      def page_recommendation_id
        @attributes['page_recommendation_id']
      end

      # Returns the site analytic ID
      #
      # @return [String]
      def site_analytic_id
        @attributes['site_analytic_id']
      end

      # Return the ID of this recommendation
      #
      # @return [String]
      def id
        (self.page_recommendation_id || self.site_analytic_id)
      end

      # Returns the ID of the site
      #
      # @return [Integer]
      def site_id
        @attributes['site_id']
      end

      # Returns the Category of the recommendation
      #
      # @todo: Create a Category model that can handle answering
      # some basic questions.
      #
      # @return [String]
      def category
        @attributes['category']
      end
      alias :category_name :category

      # Returns the Type of the recommendation
      #
      # @return [String]
      def type
        @attributes['type']
      end
      alias :type_name :type

      # Returns the Status of the recommendation
      #
      # @return [String]
      def status
        @attributes['status']
      end
      alias :status_name :status

      # Returns the Context of the recommendation
      #
      # @return [String]
      def context
        @attributes['context']
      end
      alias :context_name :context

      # Return the time of execution of this recommendation
      #
      # @return [DateTime,nil]
      def executed_at
        begin
          DateTime.parse(@attributes['executed_at'])
        rescue
          nil
        end
      end
    end
  end
end
