require File.expand_path('../../page', __FILE__)

module Nucleo
  module Models
    module FeedTypes
      class Base
        include Comparable

        # Return an instance of the Feed domain model
        #
        # @param attributes [Hash]
        #
        # @return [Nucleo::Models::Feed]
        def initialize(attributes={})
          @attributes = attributes
        end

        # Sort by the executed at attribute
        def <=>(other)
          self.executed_at <=> other.executed_at
        end

        # Returns the page, if present
        #
        # @note: We have to inject the ID back in as it's
        # not present in the `page` response.
        #
        # @return [Nucleo::Models::Page,nil]
        def page
          value = @attributes['page']

          return nil if value.nil? || value.empty?

          Nucleo::Models::Page.new(value)
        end

        # Returns true if there is an attached page
        #
        # @return [Boolean]
        def page?
          !self.page.nil?
        end

        # Returns the page recommendation ID
        #
        # @return [String]
        def page_recommendation_id
          @attributes['page_recommendation_id']
        end

        # Returns the site analytic ID
        #
        # @return [String]
        def site_analytic_id
          @attributes['site_analytic_id']
        end

        # Return the record ID
        #
        # @note: This is to support instantiation from `children`
        #
        # @return [String]
        def record_id
          @attributes['id']
        end

        # Return the ID of this recommendation
        #
        # @return [String]
        def id
          (self.page_recommendation_id || self.site_analytic_id || self.record_id)
        end

        # Returns the ID of the site
        #
        # @return [Integer]
        def site_id
          @attributes['site_id']
        end

        # Returns the Category of the recommendation
        #
        # @todo: Create a Category model that can handle answering
        # some basic questions.
        #
        # @return [String]
        def category
          @attributes['category']
        end
        alias :category_name :category

        # Returns the Type of the recommendation
        #
        # @return [String]
        def type
          @attributes['type']
        end
        alias :type_name :type

        # Returns the Status of the recommendation
        #
        # @return [String]
        def status
          @attributes['status']
        end
        alias :status_name :status

        # Returns the Context of the recommendation
        #
        # @return [String]
        def context
          @attributes['context']
        end
        alias :context_name :context

        # Return the time of execution of this recommendation
        #
        # @return [DateTime,nil]
        def executed_at
          begin
            DateTime.parse(@attributes['executed_at'])
          rescue
            nil
          end
        end

        # Returns true if this is for seo
        #
        # @return [Boolean]
        def seo_category?
          'seo' == self.category
        end

        # Returns true if this is for analytics
        #
        # @return [Boolean]
        def analytics_category?
          'analytics' == self.category
        end

        # Returns true if this is a page context
        #
        # @return [Boolean]
        def page_context?
          'page' == self.context
        end

        # Returns true if this is a site context
        #
        # @return [Boolean]
        def site_context?
          'site' == self.context
        end

        # Return the child recommendations
        #
        # @return [Nucleo::Models::Feeds]
        def children
          Nucleo::Models::Feeds.new(children_collection)
        end
        alias :issues :children

        # Returns true if there are children
        #
        # @return [Boolean]
        def children?
          self.children.any?
        end
        alias :issues? :children?

        protected
        def data
          @attributes.fetch('data', {})
        end

        def children_collection
          data.fetch('child_recommendations', [])
        end
      end
    end
  end
end
