module Nucleo
  module Models
    class Page
      # Return an instance of the Page domain model
      #
      # @param attributes [Hash]
      #
      # @return [Nucleo::Models::Page]
      def initialize(attributes={})
        @attributes = attributes
      end

      # Returns the ID of the page
      #
      # @return [String]
      def id
        @attributes['id']
      end

      # Returns the title of the page
      #
      # @return [String]
      def title
        @attributes['title']
      end

      # Returns the path of the page
      #
      # @return [String]
      def path
        @attributes['path']
      end

      # Returns the hostname of the page
      #
      # @return [String]
      def hostname
        @attributes['hostname']
      end

      # Returns the URI of the resolved URL
      #
      # @return [URI]
      def url
        URI(self.resolved_url)
      end

      # Returns the resolved URL of the page
      #
      # @return [String]
      def resolved_url
        @attributes['resolved_url'].to_s
      end

      # Returns the date the page was crawled
      #
      # @return [DateTime,nil]
      def crawled_at
        begin
          DateTime.parse(@attributes['crawled_at'])
        rescue
          nil
        end
      end

      # Returns a set of rules
      #
      # @return [Nucleo::Models::Rules]
      def rules
        Nucleo::Models::Rules.new(@attributes.fetch('rules', []))
      end

      # Returns a Feeds collection of recommendations for this page
      #
      # @return [Nucleo::Models::Feeds]
      def recommendations
        Nucleo::Models::Feeds.new(@attributes.fetch('recommendations', []))
      end

      # Returns true if there are any recommendations
      #
      # @return [Boolean]
      def recommendations?
        self.recommendations.any?
      end

      # Returns a Feeds collection of alerts for this page
      #
      # @return [Nucleo::Models::Feeds]
      def alerts
        Nucleo::Models::Feeds.new(@attributes.fetch('alerts', []))
      end

      # Returns true if there are any alerts
      #
      # @return [Boolean]
      def alerts?
        self.alerts.any?
      end

      # Returns a collection of Analytics
      #
      # @return [Array]
      def analytics
        Nucleo::Models::Feeds.new(@attributes.fetch('analytics', []))
      end

      # Returns true if there are any analytics
      #
      # @return [Boolean]
      def analytics?
        self.analytics.any?
      end

      # Returns metrics for this page
      #
      # @return [Nucleo::Models::PageMetric]
      def metrics
        Nucleo::Models::PageMetric.new(@attributes.fetch('metrics', []))
      end

      # Returns true if there are any metrics
      #
      # @return [Boolean]
      def metrics?
        self.metrics.any?
      end

      # Returns the SeoScore domain model for this page
      #
      # @return [Nucleo::Models::SeoScore]
      def seo_score
        Nucleo::Models::SeoScore.new(seo_score_params)
      end

      private
      def seo_score_params
        reference = @attributes.fetch('seo_score', {})

        {
          'score' => reference['seo_score'],
          'grade' => reference['seo_grade']
        }
      end
    end
  end
end
